/**
 * Module dependencies
 */

'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _requireSdk = require('require-sdk');

var _requireSdk2 = _interopRequireDefault(_requireSdk);

var _objectAssign = require('object-assign');

var _objectAssign2 = _interopRequireDefault(_objectAssign);

var _getYoutubeId = require('get-youtube-id');

var _getYoutubeId2 = _interopRequireDefault(_getYoutubeId);

/**
 * Create a new `player` by requesting and using the YouTube Iframe API
 *
 * @param {Object} props
 *   @param {String} url - url to be loaded
 *   @param {String} id - id of div container
 *   @param {Object} playerVars - https://developers.google.com/youtube/player_parameters
 *
 * @param {Function} cb
 */

var createPlayer = function createPlayer(props, cb) {
  var sdk = loadApi();

  var params = (0, _objectAssign2['default'])({}, props.opts, {
    videoId: (0, _getYoutubeId2['default'])(props.url)
  });

  return sdk(function (err) {
    // need to handle err better.
    if (err) {
      console.error(err);
    }

    return cb(new window.YT.Player(props.id, params));
  });
};

/**
 * Load the YouTube API
 *
 * @returns {Function}
 */

var loadApi = function loadApi() {
  var sdk = (0, _requireSdk2['default'])('https://www.youtube.com/iframe_api', 'YT');
  var loadTrigger = sdk.trigger();

  /**
   * The YouTube API requires a global ready event handler.
   * The YouTube API really sucks.
   */

  window.onYouTubeIframeAPIReady = function () {
    loadTrigger();
    delete window.onYouTubeIframeAPIReady;
  };

  return sdk;
};

/**
 * Expose `createPlayer`
 */

exports['default'] = createPlayer;
module.exports = exports['default'];